'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.mapModule = mapModule;

var _path = require('path');

function getReplaceFunc(replaceFuncPath) {
    return require((0, _path.resolve)(process.cwd(), replaceFuncPath));;
}

function mapModule(source, file, state) {
    var opts = state.opts;

    // Get replace func
    var replace = getReplaceFunc(opts.replaceFunc).default || getReplaceFunc(opts.replaceFunc);

    // If the config comes back as null, we didn't find it, so throw an exception.
    if (replace === null) {
        throw new Error('Cannot find replace function file: ' + opts.replaceFunc);
    }

    var result = replace(source, file, opts);
    if (result !== source) {
        return result;
    } else {
        return;
    }
}

exports.default = function (_ref) {
    var t = _ref.types;

    function transformRequireCall(nodePath, state) {
        if (!t.isIdentifier(nodePath.node.callee, { name: 'require' }) && !(t.isMemberExpression(nodePath.node.callee) && t.isIdentifier(nodePath.node.callee.object, { name: 'require' }))) {
            return;
        }

        var moduleArg = nodePath.node.arguments[0];
        if (moduleArg && moduleArg.type === 'StringLiteral') {
            var modulePath = mapModule(moduleArg.value, state.file.opts.filename, state);
            if (modulePath) {
                nodePath.replaceWith(t.callExpression(nodePath.node.callee, [t.stringLiteral(modulePath)]));
            }
        }
    }

    function transformImportCall(nodePath, state) {
        var moduleArg = nodePath.node.source;
        if (moduleArg && moduleArg.type === 'StringLiteral') {
            var modulePath = mapModule(moduleArg.value, state.file.opts.filename, state);
            if (modulePath) {
                nodePath.replaceWith(t.importDeclaration(nodePath.node.specifiers, t.stringLiteral(modulePath)));
            }
        }
    }

    return {
        visitor: {
            CallExpression: {
                exit: function exit(nodePath, state) {
                    return transformRequireCall(nodePath, state);
                }
            },
            ImportDeclaration: {
                exit: function exit(nodePath, state) {
                    return transformImportCall(nodePath, state);
                }
            }
        }
    };
};